﻿using SuValue.Database.Entity;
using SuValue.Database.Enum;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace SuValue.Database
{
    /// <summary>
    /// 通用数据库访问类，封装了对数据库的常见操作
    /// 创建日期：2012-11-18
    /// 修改日期：2012-11-18
    /// </summary>
    public sealed class DbUtility
    {

        #region 构造函数

        /// <summary>
        /// 构造函数
        /// </summary>
        public DbUtility()
            : this("connectionString", new List<DbParameter>(), DbProviderType.MySql)
        {


        }

        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="connectionName">数据库连接名称</param>
        public DbUtility(string connectionName)
            : this(connectionName, new List<DbParameter>(), DbProviderType.MySql)
        {

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="ConnectionString">链接数据库字符串</param>
        /// <param name="providerType"></param>
        public DbUtility(string ConnectionString, DbProviderType providerType)
        {
            _ConnectionString = ConnectionString;

            _ProviderFactory = ProviderFactory.GetDbProviderFactory(providerType);

            _DbParameters = new List<DbParameter>();

            connection = _ProviderFactory.CreateConnection();

            connection.ConnectionString = _ConnectionString;

            if (_ProviderFactory == null)
            {
                throw new ArgumentException("Can't load DbProviderFactory for given value of providerType");
            }
        }
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="connectionName">数据库连接字符串</param>
        /// <param name="parameters">参数集合</param>
        /// <param name="providerType">数据库类型枚举</param>
        public DbUtility(string connectionName, IList<DbParameter> parameters, DbProviderType providerType)
        {
            _ConnectionString = ConfigurationManager.ConnectionStrings[connectionName].ToString();

            _ProviderFactory = ProviderFactory.GetDbProviderFactory(providerType);

            _DbParameters = new List<DbParameter>();

            connection = _ProviderFactory.CreateConnection();

            connection.ConnectionString = _ConnectionString;

            if (_ProviderFactory == null)
            {
                throw new ArgumentException("Can't load DbProviderFactory for given value of providerType");
            }
        }

        #endregion

        #region 字段

        /// <summary>
        /// 数据库连接字符串名称
        /// </summary>
        private string _ConnectionString;

        /// <summary>
        /// 数据库驱动类型
        /// </summary>
        private DbProviderFactory _ProviderFactory;

        /// <summary>
        /// 参数
        /// </summary>
        private IList<DbParameter> _DbParameters;

        /// <summary>
        /// 事务
        /// </summary>
        private DbTransaction _Transaction;

        #endregion

        #region 属性

        /// <summary>
        /// 数据库连接
        /// </summary>
        public DbConnection connection;

        #endregion

        #region 事务管理

        /// <summary>
        /// 创建事务
        /// </summary>
        public void CreateTransaction()
        {
            if (_Transaction == null)
            {
                _Transaction = connection.BeginTransaction();
            }
        }

        /// <summary>
        /// 事务提交
        /// </summary>
        public void TransactionCommit()
        {

            if (_Transaction != null)
            {
                _Transaction.Commit();

                _Transaction = null;
            }
        }

        /// <summary>
        /// 事务回滚
        /// </summary>
        public void TransactionRollback()
        {
            if (_Transaction != null)
            {
                _Transaction.Rollback();

                _Transaction = null;
            }
        }

        #endregion

        #region 链接操作方法

        /// <summary>
        /// 打开数据库连接
        /// </summary>
        public void OpenConnection()
        {
            if (connection != null && connection.State != ConnectionState.Open)
                connection.Open();
        }

        /// <summary>
        /// 关闭数据库连接
        /// </summary>
        public void CloseConnection()
        {
            if (connection != null && connection.State == ConnectionState.Open)
                connection.Close();
        }

        #endregion

        #region 创建数据库参数

        /// <summary>
        /// 直接添加DbParameters参数
        /// </summary>
        /// <param name="name">名称</param>
        /// <param name="value">值</param>
        public void CreateParameter(string name, object value)
        {
            _DbParameters.Add(CreateDbParameter(name, ParameterDirection.Input, value));
        }

        /// <summary>
        /// 直接添加DbParameters参数
        /// </summary>
        /// <param name="name">名称</param>
        /// <param name="value">参数类型</param>
        /// <param name="dbtype">值</param>
        public void CreateParameter(string name, object value, DbType? dbtype)
        {
            _DbParameters.Add(CreateDbParameter(name, ParameterDirection.Input, value, dbtype));
        }

        private DbParameter CreateDbParameter(string name, ParameterDirection parameterDirection, object value)
        {
            return CreateDbParameter(name, parameterDirection, value, DbType.String);
        }

        private DbParameter CreateDbParameter(string name, ParameterDirection parameterDirection, object value, DbType? dbtype)
        {
            DbParameter parameter = _ProviderFactory.CreateParameter();
            parameter.SourceColumnNullMapping = true;
            parameter.ParameterName = name;
            parameter.Value = value;
            parameter.Direction = parameterDirection;
            if (dbtype != null) parameter.DbType = (DbType)dbtype;
            return parameter;
        }

        #endregion

        #region 创建DbCommand对象

        /// <summary>
        /// 创建一个DbCommand对象
        /// </summary>
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <param name="commandType">执行的SQL语句的类型</param>
        /// <returns></returns>
        private DbCommand CreateDbCommand(string sql, IList<DbParameter> parameters, CommandType commandType)
        {
            try
            {
                DbCommand command = _ProviderFactory.CreateCommand();

                command.CommandText = sql;

                command.CommandTimeout = 0;

                command.CommandType = commandType;

                command.Connection = connection;

                if (_Transaction != null) command.Transaction = _Transaction;

                if (!(parameters == null || parameters.Count == 0))
                {
                    foreach (DbParameter parameter in parameters)
                    {
                        command.Parameters.Add(parameter);
                    }
                }
                return command;
            }
            catch (Exception ex)
            {
                throw ex;
            }

        }

        #endregion

        #region 增删改

        /// <summary>   
        /// 对数据库执行增删改操作，返回受影响的行数。   
        /// </summary>   
        /// <param name="sql">要执行的增删改的SQL语句</param>   
        /// <returns></returns>  
        public int ExecuteNonQuerySql(string sql)
        {
            return ExecuteNonQuery(sql, _DbParameters, CommandType.Text);
        }

        /// <summary>   
        /// 对数据库执行增删改操作，返回受影响的行数。   
        /// </summary>   
        /// <param name="sql">要执行的增删改的SQL语句</param>   
        /// <param name="parameters">执行增删改语句所需要的参数</param>
        /// <returns></returns>  
        public int ExecuteNonQuerySql(string sql, IList<DbParameter> parameters)
        {
            return ExecuteNonQuery(sql, parameters, CommandType.Text);
        }

        /// <summary>   
        /// 对数据库执行增删改操作，返回受影响的行数。   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <returns></returns>  
        public int ExecuteNonQueryProc(string sql)
        {
            return ExecuteNonQuery(sql, _DbParameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 对数据库执行增删改操作，返回受影响的行数。   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <param name="parameters">执行增删改语句所需要的参数</param>
        /// <returns></returns>  
        public int ExecuteNonQueryProc(string sql, IList<DbParameter> parameters)
        {
            return ExecuteNonQuery(sql, parameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 对数据库执行增删改操作，返回受影响的行数。   
        /// </summary>   
        /// <param name="sql">要执行的增删改的SQL语句</param>   
        /// <param name="parameters">执行增删改语句所需要的参数</param>
        /// <param name="commandType">执行的SQL语句的类型</param>
        /// <returns></returns>
        private int ExecuteNonQuery(string sql, IList<DbParameter> parameters, CommandType commandType)
        {

            OpenConnection();

            using (DbCommand command = CreateDbCommand(sql, parameters, commandType))
            {
                command.CommandTimeout = 0;

                int affectedRows = 0;

                try
                {
                    affectedRows = command.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    throw ex;
                }
                finally
                {
                    _DbParameters.Clear();
                }

                return affectedRows;//影响行数
            }
        }

        #endregion

        #region 返回DataReader

        /// <summary>   
        /// 执行一个查询语句，返回一个关联的DataReader实例   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <returns></returns> 
        public DbDataReader ExecuteReaderSql(string sql)
        {
            return ExecuteReader(sql, _DbParameters, CommandType.Text);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个关联的DataReader实例   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns> 
        public DbDataReader ExecuteReaderSql(string sql, IList<DbParameter> parameters)
        {
            return ExecuteReader(sql, parameters, CommandType.Text);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个关联的DataReader实例   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <returns></returns> 
        public DbDataReader ExecuteReaderProc(string sql)
        {
            return ExecuteReader(sql, _DbParameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个关联的DataReader实例   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns> 
        public DbDataReader ExecuteReaderProc(string sql, IList<DbParameter> parameters)
        {
            return ExecuteReader(sql, parameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个关联的DataReader实例   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <param name="commandType">执行的SQL语句的类型</param>
        /// <returns></returns> 
        private DbDataReader ExecuteReader(string sql, IList<DbParameter> parameters, CommandType commandType)
        {
            using (DbCommand command = CreateDbCommand(sql, parameters, commandType))
            {
                try
                {
                    DbDataReader reader = command.ExecuteReader(CommandBehavior.CloseConnection);//CommandBehavior.CloseConnection 关闭 DataReader时关闭数据库连接
                    //DbDataReader reader = command.ExecuteReader();

                    return reader;

                }
                catch (Exception ex)
                {
                    throw ex;
                }
                finally
                {
                    _DbParameters.Clear();
                }
            }

        }

        #endregion

        #region 返回DataTable

        /// <summary>   
        /// 执行一个查询语句，返回一个包含查询结果的DataTable   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <returns></returns>
        public DataTable ExecuteDataTableSql(string sql)
        {
            return ExecuteDataTable(sql, _DbParameters, CommandType.Text);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个包含查询结果的DataTable   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns>
        public DataTable ExecuteDataTableSql(string sql, IList<DbParameter> parameters)
        {
            return ExecuteDataTable(sql, parameters, CommandType.Text);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个包含查询结果的DataTable   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <returns></returns>
        public DataTable ExecuteDataTableProc(string sql)
        {
            return ExecuteDataTable(sql, _DbParameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个包含查询结果的DataTable   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns>
        public DataTable ExecuteDataTableProc(string sql, IList<DbParameter> parameters)
        {
            return ExecuteDataTable(sql, parameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 执行一个查询语句，返回一个包含查询结果的DataTable   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <param name="commandType">执行的SQL语句的类型</param>
        /// <returns></returns>
        private DataTable ExecuteDataTable(string sql, IList<DbParameter> parameters, CommandType commandType)
        {
            using (DbCommand command = CreateDbCommand(sql, parameters, commandType))
            {
                try
                {
                    using (DbDataAdapter adapter = _ProviderFactory.CreateDataAdapter())
                    {
                        adapter.SelectCommand = command;

                        DataTable data = new DataTable();

                        adapter.Fill(data);

                        return data;
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
                finally
                {
                    _DbParameters.Clear();
                }
            }

        }

        #endregion

        #region 返回查询结果的第一行第一列

        /// <summary>   
        /// 执行一个查询语句，返回查询结果的第一行第一列   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <returns></returns>   
        public Object ExecuteScalarSql(string sql)
        {
            return ExecuteScalar(sql, _DbParameters, CommandType.Text);
        }

        /// <summary>   
        /// 执行一个查询语句，返回查询结果的第一行第一列   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>   
        /// <returns></returns>   
        public Object ExecuteScalarSql(string sql, IList<DbParameter> parameters)
        {
            return ExecuteScalar(sql, parameters, CommandType.Text);
        }

        /// <summary>   
        /// 执行一个查询语句，返回查询结果的第一行第一列   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <returns></returns>   
        public Object ExecuteScalarProc(string sql)
        {
            return ExecuteScalar(sql, _DbParameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 执行一个查询语句，返回查询结果的第一行第一列   
        /// </summary>   
        /// <param name="sql">要执行的存储过程</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>   
        /// <returns></returns>   
        public Object ExecuteScalarProc(string sql, IList<DbParameter> parameters)
        {
            return ExecuteScalar(sql, parameters, CommandType.StoredProcedure);
        }

        /// <summary>   
        /// 执行一个查询语句，返回查询结果的第一行第一列   
        /// </summary>   
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>   
        /// <param name="commandType">执行的SQL语句的类型</param>
        /// <returns></returns>   
        private Object ExecuteScalar(string sql, IList<DbParameter> parameters, CommandType commandType)
        {
            using (DbCommand command = CreateDbCommand(sql, parameters, commandType))
            {
                try
                {
                    object result = command.ExecuteScalar();

                    return result;

                }
                catch (Exception ex)
                {
                    throw ex;
                }
                finally
                {
                    _DbParameters.Clear();
                }
            }
        }

        #endregion

        #region 返回实体集合
        /// <summary>
        /// 返回对应的实体集合
        /// </summary>
        /// <typeparam name="T">需要返回集合对象</typeparam>
        /// <param name="t"></param>
        /// <returns></returns>
        public List<T> QueryForListSql<T>(T t) where T : new()
        {
            return QueryForListSql<T>(t, string.Empty);
        }
        /// <summary>
        /// 返回对应的实体集合
        /// </summary>
        /// <typeparam name="T">需要返回集合对象</typeparam>
        /// <param name="t"></param>
        /// <param name="where"></param>
        /// <returns></returns>
        public List<T> QueryForListSql<T>(T t, string where) where T : new()
        {
            return QueryForListSql<T>(t, where, string.Empty);
        }
        /// <summary>
        /// 返回对应的实体集合
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="t"></param>
        /// <param name="where"></param>
        /// <param name="orderby"></param>
        /// <returns></returns>
        public List<T> QueryForListSql<T>(T t, string where, string orderby) where T : new()
        {
            string result = ObjectToSearchSql<T>(t, where, orderby);

            return QueryForListSql<T>(result);
        }

        public Page<T> QueryForListSql<T>(T t, int pageIndex, int pageSize) where T : new()
        {
            return QueryForListSql<T>(t, pageIndex, pageSize, string.Empty);
        }

        public Page<T> QueryForListSql<T>(T t, int pageIndex, int pageSize, string where) where T : new()
        {
            return QueryForListSql(t, pageIndex, pageSize, where, string.Empty);
        }

        public Page<T> QueryForListSql<T>(T t, int pageIndex, int pageSize, string where, string orderby) where T : new()
        {
            int TotalItemCount = 0;

            string result = ObjectToSearchSql<T>(t, pageIndex, pageSize, ref TotalItemCount, where, orderby);

            Page<T> page = new Page<T>(QueryForListSql<T>(result), pageIndex, pageSize, TotalItemCount);

            page.PageIndex = pageIndex;

            page.PageSize = pageSize;



            return page;
        }

        /// <summary>
        /// 自定义分页语句
        /// </summary>
        /// <param name="totalitemSql">获得总数sql语句</param>
        /// <param name="sql">分页SQL语句</param>
        /// <returns>返回的对象中当前第几页，每页多少条都为0</returns>
        public Page<T> QueryForListSql<T>(string totalitemSql, string sql) where T : new()
        {
            int TotalItemCount = Convert.ToInt32(ExecuteScalarSql(totalitemSql));

            Page<T> page = new Page<T>(QueryForListSql<T>(sql), TotalItemCount);

            _DbParameters.Clear();

            return page;
        }

        /// <summary>
        /// 查询多个实体集合
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的查询语句</param>   
        /// <returns></returns>
        public List<T> QueryForListSql<T>(string sql) where T : new()
        {
            return QueryForListSql<T>(sql, _DbParameters);
        }

        /// <summary>
        /// 查询多个实体集合
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns>
        public List<T> QueryForListSql<T>(string sql, IList<DbParameter> parameters) where T : new()
        {
            return QueryForList<T>(sql, parameters, CommandType.Text);
        }

        /// <summary>
        /// 查询多个实体集合
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的存储过程</param>   
        /// <returns></returns>
        public List<T> QueryForListProc<T>(string sql) where T : new()
        {
            return QueryForListProc<T>(sql, _DbParameters);
        }

        /// <summary>
        /// 查询多个实体集合
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的存储过程</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns>
        public List<T> QueryForListProc<T>(string sql, IList<DbParameter> parameters) where T : new()
        {
            return QueryForList<T>(sql, parameters, CommandType.StoredProcedure);
        }

        /// <summary>
        ///  查询多个实体集合
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>   
        /// <param name="commandType">执行的SQL语句的类型</param>
        /// <returns></returns>
        private List<T> QueryForList<T>(string sql, IList<DbParameter> parameters, CommandType commandType) where T : new()
        {
            DbDataReader data = ExecuteReader(sql, parameters, commandType);

            try
            {
                return EntityReader.GetEntities<T>(data);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                data.Close();
            }
        }

        #endregion

        #region 返回实体

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <returns></returns>
        public T QueryForObjectSql<T>(T t) where T : new()
        {
            return QueryForObjectSql<T>(t, string.Empty);
        }

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <param name="sql">要执行的查询语句</param>  
        /// <param name="where">条件如（and user=‘@user’）</param>   
        /// <returns></returns>
        public T QueryForObjectSql<T>(T t, string where) where T : new()
        {
            string result = ObjectToSearchSql<T>(t, where, string.Empty);

            return QueryForObjectSql<T>(result, _DbParameters);
        }

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <param name="sql">要执行的查询语句</param>  
        /// <param name="where">条件如（and user=‘@user’）</param>  
        /// <param name="orderby">排序（order by A DEC ,B ASC）</param> 
        /// <returns></returns>
        public T QueryForObjectSql<T>(T t, string where, string orderby) where T : new()
        {
            string result = ObjectToSearchSql<T>(t, where, string.Empty);

            return QueryForObjectSql<T>(result, _DbParameters);
        }

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <param name="sql">要执行的查询语句</param>   
        /// <returns></returns>
        public T QueryForObjectSql<T>(string sql) where T : new()
        {
            return QueryForObjectSql<T>(sql, _DbParameters);
        }

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns>
        public T QueryForObjectSql<T>(string sql, IList<DbParameter> parameters) where T : new()
        {
            return QueryForObject<T>(sql, parameters, CommandType.Text);
        }

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的存储过程</param>   
        /// <returns></returns>
        public T QueryForObjectProc<T>(string sql) where T : new()
        {
            return QueryForObjectProc<T>(sql, _DbParameters);
        }

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的存储过程</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>
        /// <returns></returns>
        public T QueryForObjectProc<T>(string sql, IList<DbParameter> parameters) where T : new()
        {
            return QueryForObject<T>(sql, parameters, CommandType.StoredProcedure);
        }

        /// <summary>
        /// 查询单个实体
        /// </summary>
        /// <typeparam name="T">返回的实体集合类型</typeparam>
        /// <param name="sql">要执行的查询语句</param>   
        /// <param name="parameters">执行SQL查询语句所需要的参数</param>   
        /// <param name="commandType">执行的SQL语句的类型</param>
        /// <returns></returns>
        private T QueryForObject<T>(string sql, IList<DbParameter> parameters, CommandType commandType) where T : new()
        {
            List<T> list = QueryForList<T>(sql, parameters, commandType);

            if (list.Count > 0)
            {
                return list[0];
            }
            else
            {
                return default(T);
            }
        }

        #endregion

        #region 返回添加语句

        public string ObjectToInsertSql<T>(T t)
        {
            if (t == null) throw new ArgumentNullException("t");

            Type type = typeof(T);

            string tableName = type.Name;

            IList<ColumnAttribute> columns = GetColumnAttribute<T>(type, t);

            StringBuilder sb = new StringBuilder(" insert into  " + tableName + "  ");

            StringBuilder field = new StringBuilder("  (  ");

            StringBuilder value = new StringBuilder(" values  (  ");

            columns = columns.Where(o => o.Value != null && o.IsPrimaryKey == false && o.IsFilter == false).ToList();

            int columnsCount = columns.Count;

            for (int i = 0; i < columnsCount; i++)
            {
                ColumnAttribute item = columns[i];

                string parameterName = "@I" + _DbParameters.Count.ToString() + item.ColumnName;

                if (i == (columnsCount - 1))
                {
                    field.Append(item.ColumnName + ") ");
                    value.Append(parameterName + ") ");
                }
                else
                {
                    field.Append(item.ColumnName + " ,");
                    value.Append(parameterName + ",");
                }

                CreateParameter(parameterName, item.Value.ToString(), item.dbType);
            }

            return sb.Append(field.ToString()).Append(value.ToString()).ToString() + "; ";
        }

        #endregion

        #region 返回修改语句

        public string ObjectToUpdateSql<T>(T t)
        {
            if (t == null) throw new ArgumentNullException("执行update语句，对象为空");

            Type type = typeof(T);

            string tableName = type.Name;

            IList<ColumnAttribute> columns = GetColumnAttribute<T>(type, t);

            StringBuilder sb = new StringBuilder(" update " + tableName + "  set ");

            StringBuilder where = new StringBuilder(" where 1=1 ");

            if (columns.Where(o => o.IsPrimaryKey).Count() == 0) throw new ArgumentNullException("数据主键不能为空");

            columns = columns.Where(o => (o.Value != null || o.IsPrimaryKey) && o.IsFilter == false).ToList();

            int columnsCount = columns.Count;

            for (int i = 0; i < columnsCount; i++)
            {
                ColumnAttribute item = columns[i];

                string parameterName = "@U" + _DbParameters.Count.ToString() + item.ColumnName;

                if (item.IsPrimaryKey)
                {
                    where.Append(" and " + item.ColumnName + " =" + parameterName);
                }
                else if (i == (columnsCount - 1))
                {
                    sb.Append(item.ColumnName + " = " + parameterName);
                }
                else
                {
                    sb.Append(item.ColumnName + " = " + parameterName + " , ");
                }

                CreateParameter(parameterName, item.Value, item.dbType);
            }

            return sb.Append(where.ToString()).ToString() + "; ";
        }

        #endregion

        #region 返回删除语句

        public string ObjectToDeleteSql<T>(T t)
        {
            if (t == null) throw new ArgumentNullException("t");

            Type type = typeof(T);

            IList<ColumnAttribute> columns = GetColumnAttribute<T>(type, t);

            ColumnAttribute item = columns.Where(o => (o.IsPrimaryKey)).FirstOrDefault();

            if (item == null) throw new ArgumentNullException("没有主键，不能拼写删除语句"); //如果没有主键，不拼写SQL语句

            StringBuilder sb = new StringBuilder(" delete from  " + type.Name + " where 1 = 1 ");

            string parameterName = "@D" + _DbParameters.Count.ToString() + item.ColumnName;

            sb.Append(" and  " + item.ColumnName + " =" + parameterName);

            CreateParameter(parameterName, item.Value, item.dbType);

            return sb.ToString() + " ; ";
        }

        #endregion

        #region 返回查询语句

        public string ObjectToSearchSql<T>(T t) where T : new()
        {
            return ObjectToSearchSql<T>(t, string.Empty, string.Empty);
        }

        public string ObjectToSearchSql<T>(T t, string where, string orderby) where T : new()
        {
            if (t == null) throw new ArgumentNullException("t");

            Type type = typeof(T);

            string tableName = type.Name;

            StringBuilder sb = new StringBuilder(" select * from " + tableName + " ");

            if (!string.IsNullOrEmpty(where)) sb.Append(where);

            if (!string.IsNullOrEmpty(orderby)) sb.Append(orderby);

            return sb.ToString();

        }

        public string ObjectToSearchSql<T>(T t, int pageIndex, int pageSize, ref int TotalItemCount)
        {
            return ObjectToSearchSql<T>(t, pageIndex, pageSize, ref TotalItemCount, string.Empty, string.Empty);
        }

        public string ObjectToSearchSql<T>(T t, int pageIndex, int pageSize, ref int TotalItemCount, string where, string orderby)
        {
            if (t == null) throw new ArgumentNullException("t");

            Type type = typeof(T);

            string tableName = type.Name;

            IList<ColumnAttribute> columns = GetColumnAttribute<T>(type, t);

            StringBuilder totalItem = new StringBuilder(" select count(id) from " + tableName + " where 1=1 ");

            #region sql server 
            //StringBuilder returnsql = new StringBuilder(" select  w1.* from " + tableName + " w1 join (select top " + (pageIndex * pageSize) + " row_number() over (" + orderby + ") n, ID as wid from " + tableName + " where 1=1 ");

            //StringBuilder sql = new StringBuilder(" ) w2 on w1.id = w2.wid where 1=1 and w2.n > " + ((pageIndex - 1) * pageSize).ToString() + " ");

            //if (!string.IsNullOrEmpty(where))
            //{
            //    totalItem.Append(where);

            //    returnsql.Append(where);
            //}

            //TotalItemCount = Convert.ToInt32(ExecuteScalarSql(totalItem.ToString()));

            //returnsql.Append(sql);

            //returnsql.Append(orderby + " ;");

            //return returnsql.ToString();
            #endregion

            #region mysql

            StringBuilder returnsql = new StringBuilder(string.Format(" select  w1.* from " + tableName + " w1 where 1=1 "));

            if (!string.IsNullOrEmpty(where))
            {
                totalItem.Append(where);

                returnsql.Append(where);
            }

            TotalItemCount = Convert.ToInt32(ExecuteScalarSql(totalItem.ToString()));

            returnsql.Append(orderby);

            returnsql.Append(string.Format(" limit {0},{1}; ", (pageIndex - 1) * pageSize, pageSize));

            return returnsql.ToString();

            #endregion
        }

        #endregion

        #region 获取字段信息

        /// <summary>
        /// 获取该类型中属性与数据库字段的对应关系映射
        /// </summary>
        /// <param name="type"></param>
        public List<ColumnAttribute> GetColumnAttribute<T>(Type type, T t)
        {
            List<ColumnAttribute> propertyColumnMapping = new List<ColumnAttribute>();

            PropertyInfo[] properties = type.GetProperties();//获取所有属性

            foreach (PropertyInfo p in properties)
            {
                Attribute[] attibutes = Attribute.GetCustomAttributes(p);//获取此属性有

                ColumnAttribute cna = null;

                foreach (Attribute attribute in attibutes)
                {
                    //检查是否设置了ColumnName属性
                    if (attribute.GetType() == typeof(ColumnAttribute))
                    {
                        cna = ((ColumnAttribute)attribute);

                        break;
                    }
                }

                if (cna == null) { cna = new ColumnAttribute(p.Name, false, false); }

                if (!cna.IsFilter)
                {
                    cna.Value = EntityReader.GetValueFromObject(p.GetValue(t, null), p.PropertyType);

                    propertyColumnMapping.Add(cna);
                }

                cna.dbType = TypeToDbType(p.PropertyType);
            }

            return propertyColumnMapping;

        }

        #endregion

        private DbType? TypeToDbType(Type targetType)
        {
            switch (targetType.Name)
            {
                case "Guid": return DbType.Guid;

                default: return null;
            }
        }

    }
}
